<?php
/**
 * Google rank
 *
 * Determine actual Google rank for a given keyword and a given URL.
 * URL expected without http://
 *
 * (c) Thomas Sommerfeld, UST GmbH
 */


define("GOOGLE_HOST", "www.google.de");
define("GOOGLE_SEARCH", "/search?q=@@QUERY@@&num=@@HITS_PER_PAGE@@&start=@@START@@");

class clsGoogleRank {
  /**
   * Searchterm like entered in Google search field
   */
  var $sSearchTerm = "";
  
  /**
   * URL expected in the result
   */
  var $sURL = "";
  
  /**
   * How many search results should we crawl?
   */
  var $iSearchResults = 1000;
  
  /**
   * How many results per page?
   */
  var $iResultsPerPage = 50;
  
  /**
   * Position in searchresult
   */
  var $iPos = 0;
  
  /**
   * Real position (without duplicates)
   */
  var $iRealPos = 0;
  
  
  function clsGoogleRank($sSearchTerm = NULL, $sURL = NULL) {
    $this->setSearchTerm($sSearchTerm);
    $this->setURL($sURL);
  }
  
  function setSearchTerm($sSearchTerm = NULL) {
    if ($sSearchTerm != NULL) $this->sSearchTerm = str_replace(array(" ", "%26"), array("+", "&"), $sSearchTerm);
  }
  
  function setURL($sURL = NULL) {
    if ($sURL != NULL) $this->sURL = strtolower($sURL);
  }
  
  
  /**
   * Perform HTTP request and return result string
   */
  function doHTTP($sHost, $sGET) {
    $sAnswer = "";
    if (($sHost != "") && ($sGET != "")) {
      $h = fsockopen($sHost, 80, $iErr, $sErr);
      if ($h) {
        $sRequest = "GET $sGET HTTP/1.1\r\n";
        $sRequest .= "Host: $sHost\r\n";
        $sRequest .= "Connection: Close\r\n\r\n";
        fwrite($h, $sRequest);
        while (!feof($h)) {
          $sAnswer .= fgets($h, 1024);
        }
        fclose($h);
      } else {
        echo "Error opening socket to $sHost<br />$iErr ($sErr)<br />\n";
      }
    }
    return $sAnswer;
  }
  
  function getRank($sSearchTerm = NULL, $sURL = "") {
    $this->setSearchTerm($sSearchTerm);
    $this->setURL($sURL);

    // Reset counter
    $this->iPos = 0;
    $this->iRealPos = 0;

    $bFound  = false;
    $this->bFound = false;
    
    $sPreviousURL = "";
    
    // Request counter
    $i = 0;
    
    if ($this->iResultsPerPage < 1 || $this->iSearchResults < 1) exit;
    
    while ($i <= $this->iSearchResults && !$bFound) {
      $sGET = str_replace(array("@@QUERY@@", "@@HITS_PER_PAGE@@", "@@START@@"),
                          array($this->sSearchTerm, $this->iResultsPerPage, $i),
                          GOOGLE_SEARCH);
      $sAnswer = $this->doHTTP(GOOGLE_HOST, $sGET);
      if ($sAnswer != "") {
        // regular search results are enclosed by <div id="res">...</div>
        if (eregi('<div id=("?)res(.*)</div>', $sAnswer, $aOut)) {
          $sSearchResult = $aOut[0];
          // Each URL in the result is enclosed by <cite>...</cite>
          if (preg_match_all("|(?U)<cite>(.*)</cite>|", $sSearchResult, $aOut)) {

             // run through extracted URLs and look for search URL
             foreach ($aOut[0] as $x => $sResultURL) {
               $sResultURL = strtolower(strip_tags($sResultURL));
               $iEndPos = strpos($sResultURL, "/");
               if ($iEndPos > 0) $sResultURL = substr($sResultURL, 0, $iEndPos);
               
               // calculate result position
               $this->iPos++;
               
               if ($sResultURL === $sPreviousURL) {
                 // same URL as last result
               } else {
                 $this->iRealPos++;
               }
               $sPreviousURL = $sResultURL;
               
               // is it the searched URL?
               if ($sResultURL === $this->sURL) {
                 $bFound = true;
                 break;
               }
             }

          }
        }
      }
      $i += $this->iResultsPerPage;
    }
    
    if ($bFound) {
      $this->bFound = true;
    } else {
      $this->iPos = 0;
      $this->iRealPos = 0;
    }
    return $this->bFound;
    
  }
}

$obj = new clsGoogleRank("php blog", "www.phpblog.de");
if ($obj->getRank()) {
  echo "Site " . $obj->sURL . " found at real pos " . $obj->iRealPos . " / pos: " . $obj->iPos . " for searchterm " . $obj->sSearchTerm;
} else {
  echo "Site " . $obj->sURL . " not found for searchterm " . $obj->sSearchTerm;
}


?>
